/*----------------------------------------------------------------------+
|									|
| Copyright , Bentley Systems, Incorporated, 			|
| All rights reserved.							|
|									|
|									|
| Limited permission is hereby granted to reproduce and modify this	|
| copyrighted material provided that the resulting code is used only in |
| conjunction with Bentley Systems products under the terms of the	|
| license agreement provided therein, and that this notice is retained	|
| in its entirety in any such reproduction or modification.		|
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|									|
|   $Workfile:   FontInfo  $
|   $Revision:  $
|   	$Date:  $
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   FontInfo - FontInfo source code. |
|                                                                       |
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Include Files                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
#include    <MicroStationAPI.h>
#include    <mdl.h>
#include    <msdialog.fdf>
#include    <listmodel.fdf>
#include    <toolsubs.h>
#include    <iostream>
#include    <string.h>

#include "FontInfoCmd.h"
#include "FontInfo.h"

using namespace std;


USING_NAMESPACE_BENTLEY
USING_NAMESPACE_BENTLEY_USTN
USING_NAMESPACE_BENTLEY_USTN_ELEMENT
USING_NAMESPACE_TEXT

/*---------------------------------------------------------------------------------**//**
* @description  FontInfo_mdlCommand
* @param 	unparsed      The unparsed information sent to the command
* @bsimethod 							BSI             06/03
+---------------+---------------+---------------+---------------+---------------+------*/

struct FontTest: Bentley::Ustn::Text::FontVisitor
{
    virtual bool VisitFont (Bentley::Ustn::Text::Font &rFont)
    {
        Bentley::WString     name;
        name = rFont.GetName();
        cout << "the visiting font is " << name.GetMSWCharCP()<<endl;
        return false;
    }
};

extern "C" DLLEXPORT void DumpFontsByModel
(
char*  unparsedP
)
{
    FontTest tester;

    FontNumMap* fonts = FontManager::GetFontMap (mdlModelRef_getActive());
    fonts->VisitFonts(tester,Bentley::Ustn::Text::FINDFONT_All);
    return;

}
extern "C" DLLEXPORT void DumpFontList
(
char       *unparsedP
)
{
    FontListPtr   flip = FontManager::CreateKnownFontList (0,Bentley::Ustn::Text::FINDFONT_Normal);
    int         fontCount = flip->GetCount();
    Font*       fontPtr;
    WString     name;
    int         i;
    for (i = 0; i< fontCount; ++i)
    {
        fontPtr = flip->GetEntry(i);
        name = fontPtr->GetName();
        cout << name.GetMSWCharCP() << "is the name \n" <<endl;
    }
    return;
}

Private void    getActiveTextSettingsValue (TextParamWide& textParam, TextSizeParam& textSizeParam, DgnCacheP cache)
    {
    textSizeParam.mode        = TXT_BY_WIDTH_ASPECT | TXT_NO_TRANSFORM ;
    textSizeParam.aspectRatio = 1;

    DgnTextStylePtr astyle = TextStyleManager::GetActiveStyle ();
    if (astyle.get() == NULL)
        return;

    //astyle->GetTextParamWide (textParam, &textSizeParam.size.width, &textSizeParam.size.height, NULL, false);

    DPoint2d scale = {30,30};
   //textParam.annotationScale = dgnCache_getEffectiveAnnotationScale (cache);
   // textParam_applyAnnotationScaleFactor (&scale, &textParam, textParam.annotationScale* astyle->GetUorScale(cache));

    textSizeParam.size.width = scale.x;
    textSizeParam.size.height = scale.y;
    //textParam.exFlags.backwards = 1;
    //textParam.exFlags.upsidedown = 1;
    
    }

Private int FindFontNames 
(
FontNumMap*  fmap, 
const MSWChar*     nameP
)
{
FontP fp = NULL;
UInt32     fnum;

if(NULL  != fmap)
{
    fp = fmap->FindFontByName( &fnum, nameP);
}
return fnum;
}

extern "C" DLLEXPORT void VistFontListModel
(
char    *unparsedP
)
    {
    ListModel*      pList=NULL;
    UInt32          status;
    DialogBox*      dbP;
    TextParamWide   txtParams;
    TextSizeParam       txtP;
    DPoint2d        pScale;

    pScale.x = 30.0;
    pScale.y = 30.0;
    memset (&txtParams,0,sizeof txtParams);

    getActiveTextSettingsValue (txtParams,txtP,mdlModelRef_getCache(mdlModelRef_getActive()));

    dbP = mdlDialog_create (NULL,NULL,RTYPE_DialogBox ,DIALOGID_MDLDialog,FALSE);
    FontNumMap*   pFontMap = FontManager::GetFontMap(mdlModelRef_getActive());

    status = mdlText_getFontNameListModel (&pList,ACTIVEMODEL,dbP,FONTNAMELISTMODEL_SortByName);
    if (SUCCESS == status)
        {
        ListRow     *pRow;
        ListCell    *pCell;
        ValueDescr  pVal;
        UInt32      fontNum;

        int iCount = mdlListModel_getRowCount (pList);

        for (int i = 0;i<iCount  ;++i )
            {
            pRow = mdlListModel_getRowAtIndex (pList,i);
            pCell = mdlListRow_getCellAtIndex (pRow,0);
            mdlListCell_getValue (pCell,&pVal);
            fontNum = -1;
            Font   *pFont = (Bentley::Ustn::Text::Font*)pVal.value.voidPFormat;
            status = pFontMap->GetFontNumber (fontNum,*pFont,false);
            if (SUCCESS == status)
                {
                FindFontNames (pFontMap, pFont->GetName().GetMSWCharCP());
                cout << "the font number is" << fontNum << " and the name is " << pFont->GetName().GetMSWCharCP() <<endl;
                if (fontNum == 1025)
                    cout << "the font is 1025" << endl; 
                }
            else
                cout << "The font is not in the file" << pFont->GetName().GetMSWCharCP() << endl;
            }
        }
    return;
    }

/*---------------------------------------------------------------------------------**//**
* @description  MdlMain
* @param 	argc      The number of command line parameters sent to the application.
* @param 	argv[]    The array of strings sent to the application on the command line.
* @bsimethod 							BSI             06/03
+---------------+---------------+---------------+---------------+---------------+------*/
extern "C" DLLEXPORT  int MdlMain
(
int         argc,
char        *argv[]
)
    {
	RscFileHandle   rfHandle;

    mdlResource_openFile (&rfHandle,NULL,RSC_READONLY);

    // Map command name to function (usage: MDL COMMAND VistFontListModel)
    static  MdlCommandName cmdNames[] = 
    {
        {VistFontListModel, "VistFontListModel"  },
        {DumpFontList,"DumpFontList"},
        {DumpFontsByModel,"DumpFontsByModel"},
        0,
    };

    mdlSystem_registerCommandNames (cmdNames);

    // Map key-in to function
    static MdlCommandNumber cmdNumbers[] =
    {
		{VistFontListModel,  CMD_FONTINFO_ACTION_VISIT },
        {DumpFontList,CMD_FONTINFO_ACTION_DUMP},
        {DumpFontsByModel,CMD_FONTINFO_ACTION_BYMODEL},
        0,
    };

    mdlSystem_registerCommandNumbers (cmdNumbers);

    mdlParse_loadCommandTable (NULL);

 	return SUCCESS;
    }
